;;; Gacela, a GNU Guile extension for fast games development
;;; Copyright (C) 2017 by Javier Sancho Fernandez <jsf at jsancho dot org>
;;;
;;; This program is free software: you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation, either version 3 of the License, or
;;; (at your option) any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.


(define-module (gacela event)
  #:use-module ((sdl2 events) #:prefix sdl2:)
  #:export (process-events
            clear-events
	    quit-event?
	    key-events
	    any-key-down?
	    any-key-up?
	    key-down?
	    key-up?))


(define *current-events* '())

(define (poll-events)
  (let ((event (sdl2:poll-event)))
    (cond (event
	   (cons event (poll-events)))
	  (else
	   '()))))

(define (clear-events)
  (set! *current-events* '()))

(define (process-events)
  (set! *current-events* (append *current-events* (poll-events))))

(define (quit-event?)
  (not (null? (filter (lambda (e) (sdl2:quit-event? e)) *current-events*))))

(define (key-events)
  (filter (lambda (e) (sdl2:keyboard-event? e)) *current-events*))

(define (any-key-down?)
  (let loop ((events *current-events*))
    (if (null? events)
	#f
	(or (sdl2:keyboard-down-event? (car events))
	    (loop (cdr events))))))

(define (any-key-up?)
  (let loop ((events *current-events*))
    (if (null? events)
	#f
	(or (sdl2:keyboard-up-event? (car events))
	    (loop (cdr events))))))

(define (key-down? key)
  (let ((ke (filter
	     (lambda (e)
	       (and (sdl2:keyboard-down-event? e)
		    (equal? (sdl2:keyboard-event-key e) key)))
	     *current-events*)))
    (not (null? ke))))

(define (key-up? key)
  (let ((ke (filter
	     (lambda (e)
	       (and (sdl2:keyboard-up-event? e)
		    (equal? (sdl2:keyboard-event-key e) key)))
	     *current-events*)))
    (not (null? ke))))
